#!/usr/bin/env python

# wp_publisher.py - Desktop WordPress posting tool
# Copyright (C) 2006 V.Harishankar (http://hari.literaryforums.org)

# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, 

import xmlrpclib
import os.path
import gtk
import gtk.glade
import gobject

# Publish to the WordPress blog
def publish_wordpress (url, login, password, title, content, excerpt, categories, allow_comments, allow_pings):
	args = list ()
	content_struct = {}
	
	content_struct['title'] = title
	content_struct['description'] = content
	content_struct['mt_excerpt'] = excerpt
	content_struct['categories'] = categories
	content_struct['mt_allow_comments'] = allow_comments
	content_struct['mt_allow_pings'] = allow_pings
	
	# Set the arguments for the xml rpc call
	args.append (0)
	args.append (login)
	args.append (password)
	args.append (content_struct)
	args.append (1)
	
	# initialize the xml rpc
	my_wordpress = xmlrpclib.ServerProxy (url)
	try:
		my_wordpress.metaWeblog.newPost (args)
	except:
		dlg = gtk.MessageDialog (None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, 'Unable to connect to server. Please check your server URL, login details and your network connection settings and try again.')
		dlg.run ()
		dlg.destroy ()
		return -1
	
	# Successfully posted
	dlg_success = gtk.MessageDialog (None, gtk.DIALOG_MODAL, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, 'Successfully published post!')
	dlg_success.run ()
	dlg_success.destroy ()
	return 1

# Get the post categories
def get_post_categories ():
	cats = list ()
	item = list_categories.get_iter_first ()
	while ( item != None ):
		cats.append (list_categories.get_value (item, 0))
		item = list_categories.iter_next(item)
	return cats

# Get the post excerpt
def get_post_excerpt ():
	post_excerpt_widget = main_wnd.get_widget ('txt_post_excerpt')
	bounds = post_excerpt_widget.get_buffer().get_bounds ()
	post_excerpt = post_excerpt_widget.get_buffer().get_text (bounds[0], bounds[1])
	return post_excerpt

# Set all the fields except blog url and login name to default
def clear_defaults ():
	main_wnd.get_widget ('txt_password').set_text ('')
	main_wnd.get_widget ('txt_post_title').set_text ('')
	main_wnd.get_widget ('txt_post_content_file').set_text ('')
	main_wnd.get_widget ('txt_post_excerpt').get_buffer().set_text ('')
	main_wnd.get_widget ('txt_category').set_text ('')
	list_categories.clear ()

# The publish button is clicked
def on_btn_publish_clicked (args):
	blog_url = main_wnd.get_widget ('txt_blog_url').get_text () + '/xmlrpc.php'
	login = main_wnd.get_widget ('txt_login_name').get_text ()
	password = main_wnd.get_widget ('txt_password').get_text ()
	post_title = main_wnd.get_widget ('txt_post_title').get_text ()
	post_contents_file = main_wnd.get_widget ('txt_post_content_file').get_text ()
	# Get the post excerpt 
	post_excerpt = get_post_excerpt ()
	# Get the category list
	post_categories = get_post_categories ()
	
	# Get the comment and ping status
	if (main_wnd.get_widget ('chk_allow_comments').get_active () ):
		post_allow_comments = 0
	else:
		post_allow_comments = -1
	
	if (main_wnd.get_widget ('chk_allow_pings').get_active () ):
		post_allow_pings = 0
	else:
		post_allow_pings = -1
	
	# Check if any required fields are missing
	if (login == '' or password == '' or post_title == '' or post_contents_file == ''):
		dlg = gtk.MessageDialog (None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, 'One or more required fields is missing.')
		dlg.run ()
		dlg.destroy ()
		return
	
	# Check if the file actually exists
	if (not os.path.isfile (post_contents_file)):
		dlg = gtk.MessageDialog (None, gtk.DIALOG_MODAL, gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, 'File specified does not exist.')
		dlg.run ()
		dlg.destroy ()
		return
	
	# Get the actual post content
	fp = file (post_contents_file)
	post_content = fp.read ()
	fp.close ()
	
	# Publish to the blog
	ret = publish_wordpress (blog_url, login, password, post_title, post_content, post_excerpt, post_categories, post_allow_comments, post_allow_pings)
	
	# If successfully published, clear the fields
	if (ret == 1):
		clear_defaults ()
		

# The add category button is clicked
def on_btn_add_cat_clicked (args):
	txt_category = main_wnd.get_widget ('txt_category')
	str_val = txt_category.get_text ()
	if (str_val == ""):
		return
	list_categories.append ( [ str_val ] )
	txt_category.set_text ("")
	
# The remove category button is clicked
def on_btn_remove_clicked (args):
	cbo_category_list = main_wnd.get_widget ('cbo_category_list')
	item = cbo_category_list.get_active_iter ()
	if (item != None):
		list_categories.remove (item)

# The file browser button is clicked
def on_btn_browse_clicked (args):
	open_dlg = gtk.FileChooserDialog ('Open file', None, gtk.FILE_CHOOSER_ACTION_OPEN, (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT) )
	if ( open_dlg.run () == gtk.RESPONSE_ACCEPT):
		txt_post_content_file = main_wnd.get_widget ('txt_post_content_file')
		txt_post_content_file.set_text (open_dlg.get_filename())
	open_dlg.destroy ()
	
# Initialize the controls
def initialize_controls ():
	cbo_category_list = main_wnd.get_widget ('cbo_category_list')
	cbo_category_list.set_model (list_categories)
	
def gtk_main_quit (args):
	gtk.main_quit ()

if __name__ == "__main__":
	# Load the GTK/Glade interface
	main_wnd = gtk.glade.XML ('wp_publisher.glade', 'window_main')
	main_wnd.signal_autoconnect (globals())
	
	list_categories = gtk.ListStore (gobject.TYPE_STRING)
	initialize_controls ()
	gtk.main ()
